/*******************************************************************************
* 
* Teradek(C) 2011
* Broadband bonding functions
* 
* Questions: ilya.issenin@teradek.com
* 
*******************************************************************************/

#include <stdint.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/time.h>
	
#ifndef NETWTCP_H
#define NETWTCP_H

#define MAX_IF				16	// max number of interfaces for bonding
#define MAX_IF_NAME_LEN		16	// max length of interface name 

#define BOND_ID 0x1788abc0
#define BACK_STAT_ID 0x7868adf0
#define BOND_GET_STATS 0x00a82345
	
#define OK 0
#define NOT_OK 1

#define ARB_PRIORITY 0
#define ARB_ROUNDROBIN 1

#define NETW_ERR_TIMEOUT_MS 1000
#define SEND_TIMEOUT_S 10	// timeout for network send operation
#define MAX_WINDOW_SIZE (1024*1024)
#define MAX_FRAME_SIZE (2*1024*1024)
#define DISCONNECT_PENALTY_FRAMES 3
#define SUSPEND_LATE_FRAMES_MAX 6

struct netw_tcp_thread_args_struct
{
	void *handle;
	int conn; 
};

typedef struct
{
	char *bf1; 
	int sz1;
	char *bf2; 
	int sz2;
	int seq_num;
	int frame_type;
	uint32_t time;
	int processing;
	pthread_t threadId;
	pthread_mutex_t mutex_new_task;
	pthread_cond_t cond_new_task;
} netw_tcp_task_t;

typedef struct
{
	int late_frames;
	//int recv_frames;
	int sent_frames_susp;
	int suspended;
	int bitrate_kbps;
	uint32_t rt_delay;
} if_stats_t;

typedef struct {
	int sock[MAX_IF];
	int connected[MAX_IF];
	int bind_err[MAX_IF];
	int current_conn;
	int current_susp_conn;
	char *host;
	int port;
	int cards;
	int maxPacketSize;
	char if_name[MAX_IF][16];	// interface names for binding
	pthread_mutex_t mutex_channel_ready;
	pthread_cond_t cond_channel_ready;
	struct netw_tcp_thread_args_struct thread_args[MAX_IF];
	netw_tcp_task_t tasks[MAX_IF];
	if_stats_t stats[MAX_IF];
	uint32_t bytes_sent[MAX_IF];
	struct timeval last_stat_update;
	struct timeval bond_start_time;
	int print_stats;
	int no_headers;
	int arb_mode;
	uint32_t stream_id;
	int bond_just_started;
} netw_tcp_info_t;



int init_netw_tcp(netw_tcp_info_t *h, char *hostname, int port, char *ifnames, int maxPacketSize, int print_stats,
		int no_headers, int arb_mode);
int network_tcp_send(netw_tcp_info_t *h, char *bf1, int sz1, char *bf2, int sz2, int seq_num, 
		int frame_type, uint32_t time, int util, uint32_t len_ms);
void close_netw_tcp(netw_tcp_info_t *h);

#endif
