/*******************************************************************************
 * Copyright (c) 2010 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * This file may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/

/*******************************************************************************
 *
 * File Name    :   ServerCommon.h
 *
 * Description  :   This file contains all enums, defines, structures, functions
 *                  used commonly between all modules - API Interface Module,
 *                  Chunk Module, AMF Module, FLV Module and Handshake Module
 *
 * History:
 *  MAY/13/2010, ARJ,   Created the file.
 *  MAY/14/2010, PRP,   Started RTMP implementation
 ******************************************************************************/

#ifndef _INCLUDE_SERVER_COMMON_H
#define _INCLUDE_SERVER_COMMON_H

/*********************
 * Include Files
 *********************/
#include "ServerInclude.h"
#include "ServerBasicTypes.h"
#include "RtmpServerApi.h"
#include "ServerAmf.h"

/*********************
 * Defines and Macros
 *********************/
//#define DEBUG_LEVEL 3

/* Macros for debugging                                                       */
#define RTMP_ERROR_PRINT(format, ...) \
                fprintf (stdout, "!!!RTMP-Error!!! : " format, ## __VA_ARGS__);
#define RTMP_DEBUG_PRINT(dbgLevel, format, ...) \
            if(dbgLevel <= DEBUG_LEVEL) \
                fprintf (stdout, "!!!RTMP-Debug!!! : " format, ## __VA_ARGS__);
#define RTMP_INFO_PRINT(format, ...) \
            fprintf (stdout, "!!!RTMP-Info!!! : " format, ## __VA_ARGS__);

#define SET_ERROR_CODE(errorCode, errorNo) \
                        if(errorCode) \
                            *errorCode = errorNo;

#define FREE(pointer)   if(pointer) \
                        {\
                            free (pointer);\
                            pointer = NULL;\
                        }

/* Macros to read from buffer                                                 */
#define READ_INT16(data) ((*(data) << 8) | *(data+1))
#define READ_INT24(data) ((*(data) << 16) | (*(data+1) << 8) | *(data+2))
#define READ_INT32(data) ((*(data) << 24) | (*(data+1) << 16) | (*(data+2) << 8) | *(data+3))

/* Macros to write in buffer                                                  */
#define WRITE_INT32_LE(data, buffer) { *(buffer)    = (data & 0x000000ff);\
                                       *(buffer+1)  = (data & 0x0000ff00) >> 8;\
                                       *(buffer+2)  = (data & 0x00ff0000) >> 16;\
                                       *(buffer+3)  = (data & 0xff000000) >> 24; \
                                     }

#define WRITE_INT32(data, buffer)    { *(buffer)    = (data & 0xff000000) >> 24;\
                                       *(buffer+1)  = (data & 0x00ff0000) >> 16;\
                                       *(buffer+2)  = (data & 0x0000ff00) >> 8;\
                                       *(buffer+3)  = (data & 0x000000ff); \
                                     }


#define WRITE_INT24(data, buffer)    { *(buffer)    = (data & 0x00ff0000) >> 16;\
                                       *(buffer+1)  = (data & 0x0000ff00) >> 8;\
                                       *(buffer+2)  = (data & 0x000000ff); \
                                     }

#define WRITE_INT16(data, buffer)    { *(buffer)    = (data & 0x0000ff00) >> 8;\
                                       *(buffer+1)  = (data & 0x000000ff); \
                                     }

#define GET_CHUNK_BASIC_HDR_SIZE(chunkStrmId, headerSize) \
                                   if ((chunkStrmId > 0) && (chunkStrmId < 64))\
                                    {\
                                        headerSize =  CHUNK_BASIC_HDR0_SIZE;\
                                    }\
                                    else if (chunkStrmId <= 319)\
                                    {\
                                        headerSize = CHUNK_BASIC_HDR1_SIZE;\
                                    }\
                                    else if (chunkStrmId <= 65599)\
                                    {\
                                        headerSize = CHUNK_BASIC_HDR2_SIZE;\
                                    }\
                                    else\
                                    {\
                                        headerSize = RTMP_FAIL;\
                                    }

#define INVALID_SOCKET              (SOCKET)(~0)    /* Invalid socket value   */
#define SOCKET_ERROR                (-1)            /* Socket error value     */

#define TRUE                        0               /* Macro for True         */
#define FALSE                       1               /* Macro for False        */

#define DBG_LEVEL_1                 1               /* Macro for debug level 1*/
#define DBG_LEVEL_2                 2               /* Macro for debug level 2*/
#define DBG_LEVEL_3                 3               /* Macro for debug level 3*/
#define DBG_LEVEL_4                 4               /* Macro for debug level 4*/
#define DBG_LEVEL_5                 5               /* Macro for debug level 5*/

#define DFT_STOC_CHUNK_SIZE         128             /* Default server to client
                                                        chunk size            */
#define MAX_STOC_CHUNK_SIZE         4096            /* Max server to client
                                                        chunk size            */
#define DFT_CTOS_CHUNK_SIZE         128             /* Default client to server
                                                        chunk size            */
#define MAX_LISTEN_CLIENT           10              /* Maximum listen clients */
#define SMALL_BUFFER_SIZE           64              /* Small buffer size      */
#define MEDIUM_BUFFER_SIZE          1024            /* Medium buffer size     */
#define MAX_IP_LEN                  32              /* Length of IP           */
#define IN_ADDR_ANY                 "0.0.0.0"       /* IP address any         */
#define MAX_HEADER_COUNT            16              /* Maximum header counts  */
#define MAX_RTMP_HDR_SIZE           18              /* RTMP header size       */
#define HANDSHAKE_PACKET_SIZE       1536            /* Handshake packet size  */
#define IDENTIFIER_LEN              8               /* Length of identifier   */
#define DFT_VBITRATE                6000000         /* Default bitrate        */
#define BITRATE_VARIATION           500000          /* Bitrate variation      */
#define MAX_TIME_STAMP              0xFFFFFF        /* Max Time stamp         */
#define BW_LIMIT_TYPE               2               /* Bandwidth limit - soft */
#define INITIAL_CHUNK_STRM_ID       4               /* Start with this stream
                                                        id after play received*/
#define PROTO_CTRL_STREAM_ID        0x02            /* Stream ID for
                                                        protocol control
                                                        messages              */
#define SUPPORT_VID_ALL             0x00FF          /* Supports all video codecs*/
#define SUPPORT_VID_H264            0x0080          /* Supports H264 video    */
#define SUPPORT_SND_ALL             0x0FFF          /* Supports all audio
                                                       codecs                 */
#define SUPPORT_SND_AAC             0x0400          /* Supports AAC audio     */
#define NAL_SPS                     7               /* NAL type - SPS         */
#define NAL_PPS                     8               /* NAL type - PPS         */

#define MESSAGE_TYPE0_HDR_SIZE      11              /* Message Type 0         */
#define MESSAGE_TYPE1_HDR_SIZE      7               /* Message Type 1         */
#define MESSAGE_TYPE2_HDR_SIZE      3               /* Message Type 2         */
#define MESSAGE_TYPE3_HDR_SIZE      0               /* Message Type 3         */

#define CHUNK_BASIC_HDR0_SIZE       1               /* Header size of type 0  */
#define CHUNK_BASIC_HDR1_SIZE       2               /* Header size of type 1  */
#define CHUNK_BASIC_HDR2_SIZE       3               /* Header size of type 2  */


#define RESULT                      "_result"       /* _result                */
#define ERROR                       "_error"        /* _error                 */
#define ONSTATUS                    "onStatus"      /* onStatus               */
#define VOD                         "vod"           /* vod                    */
#define LIVE                        "live"          /* live                   */
#define ONBANDWIDTHDONE             "onBWDone"      /* onBWDone               */
#define ONFCPUBLISH                 "onFCPublish"   /* onFCPublish            */
#define RTMPSAMPLEACCESS            "|RtmpSampleAccess" /* |RtmpSampleAccess  */
#define CONNECT                     "connect"       /* connect                */
#define CREATESTREAM                "createStream"  /* createStream           */
#define PUBLISH                     "publish"       /* publish                */
#define PLAY                        "play"          /* play                   */
#define PAUSE                       "pause"         /* pause                  */
#define DELETESTREAM                "deleteStream"  /* deleteStream           */
#define APP                         "app"           /* app                    */
#define FLASHVER                    "flashVer"      /* flashVer               */
#define SWFURL                      "swfUrl"        /* swfUrl                 */
#define TCURL                       "tcUrl"         /* tcUrl                  */
#define FPAD                        "fpad"          /* fpad                   */
#define AUDIOCODECS                 "audioCodecs"   /* audioCodecs            */
#define VIDEOCODECS                 "videoCodecs"   /* videoCodecs            */
#define PAGEURL                     "pageUrl"       /* pageUrl                */
#define OBJECTENCODING              "objectEncoding"/* objectEncoding         */

#define USTREAM_PING                "ping"          /* ustream ping           */
#define USTREAM_PONG                "pong"          /* ustream pong           */
#define MODULE_INFO                 "moduleInfo"    /* moduleInfo             */
#define START_PUBLISH               "startPublish"  /* startPublish           */
#define STOP_PUBLISH                "stopPublish"   /* stopPublish            */

#define FMSVER                      "fmsVer"        /* fmsVer                 */
#define FLASH_SERVER_VERSION        "FMS/3,5,1,516" /* FMS/3,5,1,516          */
#define CAPABILITIES                "capabilities"  /* capabilities           */
#define CAPABILITY_VALUE            31.0            /* Value of capability    */
#define MODE                        "mode"          /* mode                   */
#define MODE_VALUE                  1.0             /* Value of mode          */
#define LEVEL                       "level"         /* level                  */
#define LEVEL_STATUS                "status"        /* Value of level - status*/
#define LEVEL_ERROR                 "error"         /* Value of level - error */
#define CODE                        "code"          /* code                   */
#define DESCRIPTION                 "description"   /* description            */
#define DETAILS                     "details"       /* details                */
#define CLIENT_ID                   "clientid"      /* clientid               */

#define CODE_CONN_SUCCESS_VALUE     "NetConnection.Connect.Success"
                                                    /* Netconnection Success  */
#define CODE_CONN_CLOSE_VALUE       "NetConnection.Connect.Close"
                                                    /* Netconnection Close    */
#define CODE_CONN_FAIL_VALUE        "NetConnection.Connect.Failed"
                                                    /* Netconnection Failed   */
#define CODE_STRM_PLAY_RESET        "NetStream.Play.Reset" /* Play reset      */
#define CODE_STRM_DATA_START        "NetStream.Data.Start" /* Data start      */
#define CODE_STRM_PLAY_START        "NetStream.Play.Start" /* Play start      */
#define CODE_STRM_PLAY_STOP         "NetStream.Play.Stop"  /* Play stop       */
#define CODE_STRM_NOT_FOUND         "NetStream.Play.StreamNotFound"
                                                          /* Stream not found */
#define CODE_STRM_PAUSE_NOTIFY      "NetStream.Pause.Notify"
                                                          /* Stream paused    */
#define CODE_STRM_UNPAUSE_NOTIFY    "NetStream.Unpause.Notify"
                                                          /* Stream unpaused  */

#define PLAYING_AND_RESETTING       "Playing and resetting"/* Description     */
#define STARTED_PLAYING             "Started playing"      /* Playing started */
#define STOPPED_PLAYING             "Stopped playing"      /* Playing stopped */
#define STREAM_NOT_FOUND            "Stream not found"     /* Stream not found*/
#define SUCCESS_DESC_VALUE          "Connection succeeded" /* Connection
                                                                    succeeded */
#define CLOSE_DESC_VALUE            "Connection closed" /* Connection
                                                                    closed    */
#define FAIL_DESC_VALUE             "Connection failed" /* Connection
                                                                       failed */
#define PAUSE_DESC_VALUE            "Pausing"       /* Pausing stream         */
#define UNPAUSE_DESC_VALUE          "Unpausing"     /* Unpausing stream       */
#define RTMPT_OPEN                  "open"          /* RTMPT open             */
#define RTMPT_IDLE                  "idle"          /* RTMPT close            */
#define RTMPT_SEND                  "send"          /* RTMPT send             */
#define RTMPT_CLOSE                 "close"         /* RTMPT send             */

/*********************
 * Typedefs and Enums
 *********************/

typedef enum _RTMP_SESSION_STATE
{
    SESSION_INIT_STATE = 0,                 /* Session State - Initialization */
    SESSION_READY_STATE,                    /* Session State - Ready state    */
    SESSION_INVALID_STATE                   /* Session State - Invalid state  */

} RTMP_SESSION_STATE;

typedef enum _SESSION_TYPE
{
    LIVE_SESSION,                           /* Session Type - LIVE session    */
    VOD_SESSION                             /* Session Type - VOD session     */

} SESSION_TYPE;

typedef enum _AV_FLAG
{
    AUDIO_ONLY,                             /* Flag - Audio only              */
    VIDEO_ONLY,                             /* Flag - Video only              */
    AUDIO_VIDEO_BOTH,                       /* Flag - Audio Video both        */
    INVALID                                 /* Flag - Invalid                 */

} AV_FLAG;

typedef enum _CLIENT_TYPE
{
    RTMP_CLIENT,                            /* Type of Client - RTMP          */
    RTMP_TUNNEL_CLIENT                      /* Type of Client - RTMPT         */

} CLIENT_TYPE;

typedef enum _CLIENT_STATE
{
    CLIENT_INIT_STATE,                      /* Client State - Initialization  */
    CLIENT_VERSION_SENT_DONE,               /* Client State - Version sent    */
    CLIENT_HANDSHAKE_DONE,                  /* Client State - Handshake done  */
    CLIENT_CONNECT_SENT,                    /* Client State - Connect sent    */
    CLIENT_CONNECT_DONE,                    /* Client State - Connect done    */
    CLIENT_CREATESTREAM_SENT,               /* Client State - Createstream sent*/
    CLIENT_CREATESTREAM_DONE,               /* Client State - Createstream done*/
    CLIENT_PUBLISH_SENT,               		/* Client State - Publish sent*/
    CLIENT_PUBLISH_DONE,               		/* Client State - Publish done*/
    CLIENT_PLAY_DONE,                       /* Client State - Play done       */
    CLIENT_PAUSE_DONE,                      /* Client State - Pause done      */
    CLIENT_STOP_DONE,                       /* Client State - Stop done       */
    CLIENT_DELETESTREAM_DONE,               /* Client State - DeleteStream done*/
    CLIENT_DESTROY_STATE,                   /* Client State - Destroying      */
    CLIENT_INVALID_STATE                    /* Client State - Invalid         */

} CLIENT_STATE;

typedef enum _CLIENT_RTMPT_STATE
{
    CLIENT_RTMPT_OPEN_DONE,                 /* RTMPT Client state - Open done */
    CLIENT_RTMPT_IDLE_DONE,                 /* RTMPT Client state - Idle done */
    CLIENT_RTMPT_SEND_DONE,                 /* RTMPT Client state - Send done */
    CLIENT_RTMPT_CLOSE_DONE,                /* RTMPT Client state - Close done*/
    CLIENT_RTMPT_INVALID_STATE              /* RTMPT Client state - Invalid   */

} CLIENT_RTMPT_STATE;

typedef enum _PACKET_STATUS
{
    PACKET_INCOMPLETE,                      /* Status of packet - Incomplete  */
    PACKET_COMPLETE,                        /* Status of packet - Complete    */
    PACKET_PROCESSED                        /* Status of packet - Processed   */

} PACKET_STATUS;

typedef enum _MESSAGE_HEADER_TYPE
{
    MSG_HEADER_TYPE0  = 0,                  /* Message Header Type 0          */
    MSG_HEADER_TYPE1  = 1,                  /* Message Header Type 1          */
    MSG_HEADER_TYPE2  = 2,                  /* Message Header Type 2          */
    MSG_HEADER_TYPE3  = 3                   /* Message Header Type 3          */

} MESSAGE_HEADER_TYPE;

typedef enum _BASIC_HEADER_TYPE
{
    BASIC_HEADER_TYPE0,                     /* Basic Header Type 0            */
    BASIC_HEADER_TYPE1,                     /* Basic Header Type 1            */
    BASIC_HEADER_TYPE2                      /* Basic Header Type 2            */

} BASIC_HEADER_TYPE;

typedef enum _RTMP_MESSAGE_TYPE
{
    AMF0_SET_CHUNK_SIZE             = 0x1,  /* Message - Set chunk size       */
    AMF0_ABORT_MESSAGE              = 0x2,  /* Message - Abort message        */
    AMF0_ACKNOWLEDGEMENT            = 0x3,  /* Message - Acknowledgement      */
    AMF0_USER_CONTROL_MESSAGE       = 0x4,  /* Message - User control message */
    AMF0_WINDOW_ACKNOWLEDGEMENT_SIZE= 0x5,  /* Message - Win Ack size         */
    AMF0_SET_PEER_BANDWIDTH         = 0x6,  /* Message - Set peer bandwidth   */
    AMF0_AUDIO_MESSAGE              = 0x8,  /* Message - Audio message        */
    AMF0_VIDEO_MESSAGE              = 0x9,  /* Message - Video message        */
    AMF0_DATA_MESSAGE               = 0x12, /* Message - Data message         */
    AMF0_SHARED_OBJECT_MESSAGE      = 0x13, /* Message - Shared object message*/
    AMF0_COMMAND_MESSAGE            = 0x14, /* Message - Command message      */
    AMF0_AGGREGATE_MESSAGE          = 0x16  /* Message - Aggregate message    */

} RTMP_MESSAGE_TYPE;

typedef enum _RTMP_EXACT_MSG_TYPE
{
    COMMAND_MSG_RESULT,                	/* Command Message - RESULT           */
    COMMAND_MSG_ERROR,                	/* Command Message - ERROR            */
    COMMAND_MSG_CONNECT,                /* Command Message - CONNECT          */
    COMMAND_MSG_ONSTATUS,				/* Command Message - ONSTATUS		  */
    COMMAND_MSG_CREATE_STREAM,          /* Command Message - CREATESTREAM     */
    COMMAND_MSG_FCPUBLISH,          	/* Command Message - FCPUBLISH	      */
    COMMAND_MSG_ONFCPUBLISH,          	/* Command Message - ONFCPUBLISH	  */
    COMMAND_MSG_PUBLISH,          		/* Command Message - PUBLISH	      */
    COMMAND_MSG_PLAY,                   /* Command Message - PLAY             */
    COMMAND_MSG_PAUSE,                  /* Command Message - PAUSE            */
    COMMAND_MSG_ON_BW_DONE,             /* Command Message - ON BANDWIDTH DONE*/
    COMMAND_MSG_PLAY_RESET_DONE,        /* Command Message - PLAY RESET       */
    COMMAND_MSG_PLAY_START_DONE,        /* Command Message - PLAY START       */
    COMMAND_MSG_PLAY_STOP_DONE,         /* Command Message - PLAY STOP        */
    COMMAND_MSG_STREAM_NOT_FOUND,       /* Command Message - STREAM NOT FOUND */
    COMMAND_MSG_CONNECTION_FAILED,      /* Command Message - UNSUPPORTED      */
    COMMAND_MSG_DELETESTREAM,           /* Command Message - DELETESTREAM     */

    // Ustream commands
    COMMAND_MSG_USTREAM_PING,           /* Command Message - USTREAM PING     */
    COMMAND_MSG_USTREAM_PONG,           /* Command Message - USTREAM PONG     */
    COMMAND_MSG_START_PUBLISH,          /* Command Message - START_PUBLISH    */
    COMMAND_MSG_STOP_PUBLISH,           /* Command Message - STOP_PUBLISH     */

    USR_CTRL_MSG_STREAM_BEGIN,          /* User Control Message - STREAM BEGIN*/
    USR_CTRL_MSG_STREAM_EOF,            /* User Control Message - STREAM EOF  */
    USR_CTRL_MSG_STREAM_DRY,            /* User Control Message - STREAM DRY  */
    USR_CTRL_MSG_SET_BUFFER_LEN,        /* User Control Message - SET BUFFER LEN  */
    USR_CTRL_MSG_STREAM_IS_RECORDED,    /* User Control Message - STREAM IS RECORDED    */
    USR_CTRL_MSG_PING_REQUEST,          /* User Control Message - PING REQUEST*/
    USR_CTRL_MSG_PING_RESPONSE,         /* User Control Message -PING RESPONSE*/


    PROTO_CTRL_MSG_SET_CHUNK_SIZE,      /* Protocol Control Message - SET CHUNK SIZE    */
    PROTO_CTRL_MSG_ABORT,               /* Protocol Control Message - ABORT   */
    PROTO_CTRL_MSG_ACKNOWLEDGEMENT,     /* Protocol Control Message - ACKNOWLEDGEMENT   */
    PROTO_CTRL_MSG_WIN_ACK_SIZE,        /* Protocol Control Message - WIN ACK SIZE      */
    PROTO_CTRL_MSG_SET_PEER_BANDWIDTH,  /* Protocol Control Message - SET PEER BANDWIDTH    */

    DATA_MSG_DATA_START,                /* Data Message - Data Start          */
    DATA_MSG_METADATA,                  /* Data Message - MetaData            */
    DATA_MSG_RTMP_SAMPLE_ACCESS,        /* Data Message - RTMP Sample Access  */
    MEDIA_MSG_VIDEO,                    /* Media Message - Video              */
    MEDIA_MSG_AUDIO,                    /* Media Message - Audio              */
    NOT_SUPPORTED_MSG_TYPE              /* Not supported message type         */

} RTMP_EXACT_MSG_TYPE;

typedef enum _USR_CTRL_MSGS
{
    STREAM_BEGIN        = 0x00,         /* STREAM BEGIN                       */
    STREAM_EOF          = 0x01,         /* STREAM EOF                         */
    STREAM_DRY          = 0x02,         /* STREAM DRY                         */
    SET_BUFFER_LEN      = 0x03,         /* SET BUFFER LENGTH                  */
    STREAM_IS_RECORDED  = 0x04,         /* STREAM IS RECORDED                 */
    PING_REQUEST        = 0x06,         /* PING REQUEST                       */
    PING_RESPONSE       = 0x07          /* PING RESPONSE                      */

} USR_CTRL_MSGS;

typedef enum _TUNNEL_RTMP_METHOD
{
    RTMP_METHOD_OPEN = 0,               /* RTMPT METHOD - OPEN                */
    RTMP_METHOD_IDLE,                   /* RTMPT METHOD - IDLE                */
    RTMP_METHOD_SEND,                   /* RTMPT METHOD - SEND                */
    RTMP_METHOD_CLOSE,                  /* RTMPT METHOD - CLOSE               */
    RTMP_METHOD_INVALID                 /* RTMPT METHOD - INVALID             */

} TUNNEL_RTMP_METHOD;

typedef enum _INTERNAL_ERROR_CODES
{
    INVALID_STREAM_NAME = -111,         /* Invalid stream name                */
    UNSUPPORTED_MESSAGE_TYPE            /* Message not supported              */

} INTERNAL_ERROR_CODES;

typedef enum _PublishLogLevel {
    PUBLISH_LOG_OFF = 0,
    PUBLISH_LOG_LEVEL_1,
    PUBLISH_LOG_LEVEL_2,
    PUBLISH_LOG_LEVEL_3
}PublishLogLevel;

typedef enum _PublishState {
    PUBLISH_STATE_READY = 0,
    PUBLISH_STATE_CONNECTING,
    PUBLISH_STATE_CONNECTED,
    PUBLISH_STATE_FORCE_RECONNECT,
    PUBLISH_STATE_DISCONNECTED,
    PUBLISH_STATE_RECONNECTING,
    PUBLISH_STATE_CLOSE,
    PUBLISH_STATE_INVALID
}PublishState;

/*********************
 *    Structures
 *********************/

typedef struct _RtmpServerLib       RtmpServerLib;
typedef struct _RtmpServer          RtmpServer;
typedef struct _RtmpSession         RtmpSession;
typedef struct _RtmpClient          RtmpClient;
typedef struct _RtmpCodecInfo       RtmpCodecInfo;
typedef struct _RtmpVdCodecInfo     RtmpVdCodecInfo;
typedef struct _RtmpAdCodecInfo     RtmpAdCodecInfo;
typedef struct _TunnelRTMPReqMsg    TunnelRTMPReqMsg;
typedef struct _TunnelRTMPResMsg    TunnelRTMPResMsg;
typedef struct _RtmpRecvStrmPkt     RtmpRecvStrmPkt;
typedef struct _RtmpSendStrmPkt     RtmpSendStrmPkt;
typedef struct _HTTPMsgHeaders      HTTPMsgHeaders;
typedef struct _RtmpData            RtmpData;
typedef struct _PublishServer       PublishServer;
typedef struct _PublishError        PublishError;
typedef struct _PublishStatus       PublishStatus;
typedef struct _PublishAuthentication       PublishAuthentication;
typedef struct _RtmpPacketBuffer    RtmpPacketBuffer;
typedef struct _RtmpSenderQueue     RtmpSenderQueue;
typedef struct _RtmpSenderMonitor   RtmpSenderMonitor;
typedef struct _RtmpSenderMonitorEntry      RtmpSenderMonitorEntry;

struct _PublishAuthentication
{
    INT32 required;
    UINT8 authmod[1024];
    UINT8 reason[1024];
    UINT8 user[1024];
    UINT8 password[1024];
    UINT8 salt[1024];
    UINT8 challenge[1024];
    UINT8 response[1024];
    UINT8 opaque[1024];
};

struct _PublishServer
{
    UINT8 ip[1024];
    UINT8 file[1024];
    INT32 port;
    UINT8 username[1024];
    UINT8 password[1024];
    UINT8 streamName[1024];
    UINT8 user_agent[1024];
    UINT8 copyright[1024];
    UINT8 device[1024];

    UINT8 platform[1024];
    UINT8 s_key[1024];
    UINT8 prefix[1024];

    PublishState state;
    PublishLogLevel logLevel;
    INT32 compatibilityMode;
    PublishAuthentication *auth;
};

struct _PublishError
{
    INT32 code;
    UINT8 level[1024];
    UINT8 shortStr[1024];
    UINT8 longStr[1024];
};

struct _PublishStatus
{
    UINT8 level[1024];
    UINT8 shortStr[1024];
    UINT8 longStr[1024];
};

struct _RtmpPacketBuffer
{
    SHANDLE rtmpSession; // Pointer to rtmp Session
    UINT8 *dataBuff; // Pointer to audio/video data
    INT32 dataSize; // Size of audio/video data
    INT32 deviceFd; // Zero copy device fd
    VOID *vZeroPtr; // Zero copy pointer
    UINT64 prevTimeStamp; // time stamp from the encoder
    UINT64 timeStampDiff; // difference between last sent frame of the same type and this packet
    RTMP_EXACT_MSG_TYPE exactMsgTypeId; // msg type
    STREAM_TYPE strmType; // data type (audio/video)

    // Doubly linked list
    RtmpPacketBuffer *prevPacket;
    RtmpPacketBuffer *nextPacket;
};

struct _RtmpSenderMonitorEntry
{
    STREAM_TYPE                 type;
    struct timeval              time;
    INT32                       size;

    INT32                       currentTotal;
};

struct _RtmpSenderMonitor
{
    RtmpSenderMonitorEntry      *queue;
    INT32                       tail;
    INT32                       head;

    INT32                       size;
    INT32                       total;
    INT32                       average;

    INT32                       sumTotal;

    struct timeval              lastUpdate;
};

struct _RtmpSenderQueue
{
    SHANDLE rtmpSession;

    /* previous audio/video timestamp (to calculate deltas) */
    UINT64                      prevAudioTimeStamp;
    UINT64                      prevVideoTimeStamp;

    /* RTMP timestamps for audio/video packets */
    UINT64                      lastSentVideoTimeStamp;
    UINT64                      lastSentAudioTimeStamp;

    /* Video Packet buffer */
    RtmpPacketBuffer            *VideoBuffer; // Pointer to static mem
    INT32                       videoHead; //
    INT32                       videoTail;
    RtmpPacketBuffer            *VideoBufferPtr; // Moving pointer of rotating buffer
    RtmpPacketBuffer            *VideoBufferEndPtr;
    INT32                       VideoBufferSize; // Current size

    /* Audio Packet buffer */
    RtmpPacketBuffer            *AudioBuffer; // Pointer to static mem
    INT32                       audioHead;
    INT32                       audioTail;
    RtmpPacketBuffer            *AudioBufferPtr;
    RtmpPacketBuffer            *AudioBufferEndPtr;
    INT32                       AudioBufferSize;

    UINT8                       highWaterMark;
    UINT64                      highWaterMarkTimeStamp;
    struct timeval              highWaterMarkTime;


    RtmpSenderMonitor           *bandwidthMonitor;
    SHANDLE                     avcencHandle;

    pthread_t                   ThreadId;
    pthread_mutex_t             ThreadLock;
    pthread_cond_t              ThreadReady;
};

struct _RtmpServerLib
{
    INT32                       ServerListCount;/* Nomber of server in list   */
    RtmpServer                  *ServerList;    /* Pointer to server list     */
    RtmpServer                  *ServerListEndPtr;/* Pointer to server list   */
    LOCK_HANDLE                 ServerListLock; /* Server list lock           */
};

struct _RtmpServer
{
    UINT8                       ServerIP[MAX_IP_LEN];/* Server IP address     */
    INT32                       WinAckSize;     /* Window ack. size           */
    INT32                       MaxVideoBitrate;/* Maximum video bitrate      */
    INT32                       StoCChunkSize;  /* Server to client chunk size*/
    INT32                       PreferedStoCChunkSize;  /* Server to client chunk size*/

    UINT16                      RtmpPort;       /* Rtmp server port           */
    SOCKET                      RtmpSocket;     /* Socket to accept Rtmp client
                                                   connection                 */
    THREAD_HANDLE               RtmpSrvThrdId;  /* Rtmp server thread ID      */

    UINT16                      RtmptPort;      /* Rtmpt server port          */
    SOCKET                      RtmptSocket;    /* Socket to accept Rtmpt client
                                                   Connection                 */
    THREAD_HANDLE               RtmptSrvThrdId; /* Rtmpt server thread ID     */

    /* Live session List                                                      */
    INT32                       LiveSessionCount;/* Number of live session    */
    LOCK_HANDLE                 LiveSessionListLock;/* Live session list lock */
    RtmpSession                 *LiveSessionList;/* Live session list         */
    RtmpSession                 *LiveSessionListEndPtr;/* Live session list   */

    RtmpServerLib               *ServerLibPtr;  /* Pointer to Rtmp server lib */

    RtmpServer                  *PrevServer;    /* Next server in list        */
    RtmpServer                  *NextServer;    /* Previous server in list    */
};

struct _RtmpSession
{
    UINT8                       *SessionName;   /* Name of session            */
    SESSION_TYPE                SessionType;    /* Type of session            */
    RTMP_SESSION_STATE          SessionState;   /* State of session           */
    AV_FLAG                     AvFlag;         /* Audio video flag           */
    RtmpCodecInfo               *CodecInfo;     /* Information of codec       */

    INT32                       ClientListCount;/* Number of connected clients*/
    RtmpClient                  *ClientList;    /* List of connected clients  */
    RtmpClient                  *ClientListEndPtr;/* List of connected clients*/
    LOCK_HANDLE                 ClientListLock; /* Client list lock           */

    RtmpServer                  *RtmpServerPtr; /* Pointer to Rtmp server     */

    INT32                       useSender;      /* Flag to use the sender queue */
    INT32                       tick_120;
    RtmpSenderQueue             *rtmpQueue;     /* Sender queue */

    RtmpSession                 *NextSession;   /* Next session in list       */
    RtmpSession                 *PrevSession;   /* Previous session in list   */
};

struct _RtmpCodecInfo
{
    VIDEO_CODEC_TYPE            VideoType;      /* Type of video codec        */
    INT32                       VdBitrate;      /* Video bitrate              */
    INT32                       VdConfigDataSize;/* Size of video config data */
    UINT8                       *VdConfigData;  /* Pointer to video config    */
    RtmpVdCodecInfo             *VdCodecInfo;   /* Video codec information    */

    AUDIO_CODEC_TYPE            AudioType;      /* Type of audio codec        */
    INT32                       AdBitrate;      /* Audio bitrate              */
    INT32                       AdConfigDataSize;/* Size of audio config data */
    UINT8                       *AdConfigData;  /* Pointer to audio config    */
    RtmpAdCodecInfo             *AdCodecInfo;   /* Audio codec information    */

    UINT8                       *MetaData;      /* Pointer to metadata info   */
    INT32                       MetaDataSize;   /* Size of metadata info      */
};

struct _RtmpAdCodecInfo
{
    INT32                       SampleRate;     /* Sample rate of audio       */
    INT32                       NoOfChannles;   /* Number of channels         */
    INT32                       Bitrate;        /* Bitrate of audio           */
    UINT8                       AacConfigPkt [4];/* Pointer to AAC config pkt  */
    INT32                       AacConfigPktSize;/* Size of AAC config packet */
};


struct _RtmpVdCodecInfo
{
    INT32                       Height;         /* Height of video            */
    INT32                       Width;          /* Width of video             */
    INT32                       Bitrate;        /* Bitrate of video           */
    INT32                       ProfileId;      /* Profile Id of video        */
    INT32                       ProfileCompat;  /* Profile compatibility      */
    INT32                       LevelId;        /* Level Id of video          */
    INT32                       fr1001;         /* Fractional framerate flag  */
    INT32                       fr;             /* Framerate                  */
    UINT8                       *AvccBox;       /* Pointer to avcC data       */
    INT32                       AvccBoxSize;    /* Size of avcC data          */
};

struct _RtmpClient
{
    UINT8                       *ClientIP;      /* IP address of
                                                            connected client  */
    CLIENT_TYPE                 ClientType;     /* Type of client connected   */
    SOCKET                      ClientSocket;   /* Socket of connected client */
    UINT16                      ClientPort;     /* Port of connected client   */
    CLIENT_STATE                ClientState;    /* State of client            */
    CLIENT_RTMPT_STATE          ClientRtmptState;/* RTMPT state of client     */
    THREAD_HANDLE               ClientThreadID; /* Thread ID of client        */
    INT32                       WinAckSize;     /* Window ack. size           */
    BOOL                        PauseFlag;      /* Flag to indicate pause
                                                   and resume                 */
    UINT64                      ClientStartTime;/* Up Time of client          */
    INT32                       CtoSChunkSize;  /* Client to Server chunk size*/
    UINT8                       ClientId [IDENTIFIER_LEN+1];/* Client ID for
                                                            RTMP session      */
    UINT8                       *ReqSessionName;/* Name of request session    */

    INT32                       NetConnStrmId;  /* NetConnection - Stream Id  */
    INT32                       NetStreamStrmId;/* NetStream - Streamd Id     */
    INT32                       NetStreamVdChunkId;/* Video stream chunk Id   */
    INT32                       NetStreamAdChunkId;/* Audio stream chunk Id   */

    TunnelRTMPReqMsg            *TunnelReqMsg;  /* RTMPT request message      */
    TunnelRTMPResMsg            *TunnelResMsg;  /* RTMPT response message     */

    INT32                       RecvStrmListCount;/* Total recv pkts in list  */
    RtmpRecvStrmPkt             *RecvStrmList;  /* Recv packet list           */
    RtmpRecvStrmPkt             *RecvStrmListEndPtr;/* Pointer to last packet */
    LOCK_HANDLE                 RecvStrmListLock;/* Lock for recv packet list */

    INT32                       SendStrmListCount;/* Total send pkts in list  */
    RtmpSendStrmPkt             *SendStrmList;  /* Send packet list           */
    RtmpSendStrmPkt             *SendStrmListEndPtr;/* Pointer to last packet */
    LOCK_HANDLE                 SendStrmListLock;/* Lock for send packet list */

    RtmpSession                 *RtmpSessionPtr;/* Pointer to rtmp session    */
    RtmpServer                  *RtmpServerPtr; /* Pointer to rtmp server     */

    RtmpClient                  *NextClient;    /* Pointer to next client     */
    RtmpClient                  *PrevClient;    /* Pointer to prev client     */

    PublishServer				*PublishPtr;

    PublishError				*PublishErrorPtr;
    PublishStatus				*PublishStatusPtr;

};

struct _RtmpRecvStrmPkt
{
    INT32                       ChunkStreamId;  /* Chunk stream identifier    */

    INT32                       PktCountId;     /* Packet count identifier    */
    PACKET_STATUS               PktStatus;      /* Packet status              */
    INT32                       PayloadBytesReceived; /* Paylod bytes received*/
    MESSAGE_HEADER_TYPE         ChunkMsgHdrType;/* Type of message header     */
    BASIC_HEADER_TYPE           ChunkBasicHdrType; /* Type of basic header    */

    RTMP_MESSAGE_TYPE           TypeId;         /* Type id of received packet */
    UINT8                       TimeStamp [3];  /* Timestamp of received pkt  */
    UINT8                       TimeStampDelta [3]; /* Timestamp delta        */
    UINT8                       ExtTimeStamp [4];/* Extented TS of recv pkt   */
    UINT8                       StrmId [4];     /* Streamd Id of recv pkt     */
    INT32                       PayloadSize;    /* Size of payload            */
    UINT8                       *PayloadData;   /* Pointer to payload data    */

    union
    {
        AmfObj                  *Object;        /* Amfobject type payload data*/
        UINT8                   *MiscData;      /* Misc payload data          */
        EventBuff               Event;          /* Event type payload data    */
    }ParsedPayload;

    RtmpRecvStrmPkt             *NextStrmPkt;   /* Pointer to next stream pkt */
    RtmpRecvStrmPkt             *PrevStrmPkt;   /* Pointer to prev stream pkt */
};

struct _RtmpSendStrmPkt
{
    INT32                       ChunkStreamId;   /* Chunk stream identifier   */

    RTMP_MESSAGE_TYPE           TypeId;         /* Type id of received packet */
    RTMP_EXACT_MSG_TYPE         ExactTypeId;    /* Exact message type Id      */

    UINT64                      PresTimestamp;  /* Present system time        */
    UINT8                       TimeStamp[3];   /* Timestamp of send stream
                                                   pkt                        */
    UINT8                       TimeStampDelta[3];/* Timestamp delta          */
    UINT8                       ExtTimeStamp[4];/* Extented TS of send stream
                                                   pkt                        */
    UINT8                       ExtTimeStampDelta[4];/* Extended TS delta of
                                                   send stream packet         */
    UINT8                       StrmId[4];      /* Streamd Id of send stream
                                                   pkt                        */
    INT32                       PayloadSize;    /* Size of payload            */
    UINT8                       *PayloadData;   /* Pointer to payload data    */

    struct _RtmpSendStrmPkt     *NextStrmPkt;   /* Pointer to next stream pkt */
    struct _RtmpSendStrmPkt     *PrevStrmPkt;   /* Pointer to prev stream pkt */
};

struct _RtmpData
{
    INT32                       Size;           /* Size of data               */
    UINT8                       *Data;          /* Pointer to data            */
};

struct _HTTPMsgHeaders
{
    UINT8                       *HdrName;       /* Message Header Name        */
    UINT8                       *HdrValue;      /* Message Header Value       */

};

struct _TunnelRTMPResMsg
{
    INT32                       StatusCode;     /* Status code                */
    INT32                       SeqNumber;      /* Sequence number            */
    INT32                       HeadersCount;   /* Number of headers          */
    UINT8                       *ReasonPhrase;  /* Reason of status           */
    UINT8                       *HttpVersion;   /* Http version               */
    HTTPMsgHeaders              MsgHeaders [MAX_HEADER_COUNT]; /* Headers     */
};

struct _TunnelRTMPReqMsg
{
    INT32                       SeqNumber;      /* Sequence number            */
    INT32                       HeadersCount;   /* Number of headers          */
    UINT8                       *MsgURI;        /* Message URI                */
    UINT8                       *HttpVersion;   /* Http version               */
    TUNNEL_RTMP_METHOD          Method;         /* RTMPT method name          */
    HTTPMsgHeaders              MsgHeaders [MAX_HEADER_COUNT];/* Headers      */

    INT32                       NoOfBytesReceived; /* No. of bytes received   */
    INT32                       ContentLength;  /* Content length             */
};


/************************
 * Function declarations
 ************************/

/* Thread for accepting connections from clients on RTMP                      */
INT32 AcceptRtmpConnectionThread (VOID *threadArgs);

/* Thread for accepting connections from clients on RTMPT                     */
INT32 AcceptRtmptConnectionThread (VOID *threadArgs);

/* Function to validate server handle                                         */
INT32 ValidateRtmpServerHandle (RtmpServerLib *rtmpServerLib, RtmpServer *rtmpServer);

/* Function to validate session handle                                        */
INT32 ValidateRtmpSessionHandle (RtmpServerLib *rtmpServerLib, RtmpSession *rtmpSession);

/* Function to validate stream name as received from client                   */
INT32 ValidateRtmpStreamName (RtmpClient *clientPtr, INT32 streamNameSize,
                              UINT8 *streamName, RtmpSession **validSessionPtr);

/* Function to check whether session name already exists or not               */
INT32 IsSessionNameExists (RtmpServer *rtmpServer, UINT8 *sessionName);

/* Function to check whether server already exists or not                     */
RtmpServer* IsServerExists (RtmpServerLib *rtmpServerLib,
                            UINT8 *serverIP, UINT16 rtmpPort, UINT16 rtmptPort);

/* Function to read RTMP Message                                              */
RtmpRecvStrmPkt* ReadRtmpMessage (RtmpClient *clientPtr);

/* Function to check whether recv packet exists or not in list                */
INT32 IsRecvStrmPktExists (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt);

/* Function to decide which type of message header to be generated            */
MESSAGE_HEADER_TYPE DecideMsgHdrType (RtmpSendStrmPkt *curSendPkt,
                                      RtmpSendStrmPkt *lastSendPkt);

/* Function to get received packet from list                                  */
RtmpRecvStrmPkt* GetRecvPktFromList (RtmpClient *clientPtr, INT32 chunkStrmId);

/* Function to get send packet from list                                      */
RtmpSendStrmPkt *GetSendPktFromList (RtmpClient *clientPtr, INT32 chunkStrmId);

/* Function to add Rtmp receive packet in list                                */
INT32 AddRecvPkt (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt);

/* Function to add Rtmp send packet in list                                   */
INT32 AddSendPkt (RtmpClient *clientPtr, RtmpSendStrmPkt *sendStrmPkt);

/* Function to remove Rtmp receive packet in list                             */
INT32 RemoveRecvPkt (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt);

/* Function to remove Rtmp send packet in list                                */
INT32 RemoveSendPkt (RtmpClient *clientPtr, RtmpSendStrmPkt *sendStrmPkt);

/* Function to create new rtmp packet                                         */
RtmpRecvStrmPkt* CreateNewRtmpPkt (RtmpClient *clientPtr,
                                   UINT32 dataBuffSize, UINT8 *dataBuff,
                                   INT32 payloadSizeInHdr, INT32 chunkMsgHdrType,
                                   BASIC_HEADER_TYPE chunkBasicHdrType,
                                   INT32 chunkStrmId);

/* Function to get payload header size from last same stream id packet        */
INT32 GetPayloadHdrSize (RtmpClient *clientPtr, INT32 chunkStrmId);

/* Function to get message type id from last same stream id packet            */
RTMP_MESSAGE_TYPE GetMsgTypeId (RtmpClient *clientPtr, INT32 chunkStrmId);

/* Function to parse received request message                                 */
INT32 ParseRequestMsg (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt,
                       RTMP_EXACT_MSG_TYPE *ExactMsgTypeId);

/* Function to parse command request message                                  */
INT32 ParseCmdMsgReq (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt,
                        RTMP_EXACT_MSG_TYPE *exactMsgTypeId);

/* Function to parse user control request message                             */
INT32 ParseUsrCtrlMsgReq (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt,
                        RTMP_EXACT_MSG_TYPE *exactMsgTypeId);

/* Function to process request message, generate and send response message    */
INT32 ProcessSend (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt,
                    RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to generate rtmp response message                                 */
RtmpSendStrmPkt* GenerateRtmpMsg (RtmpClient *clientPtr,
                                  RtmpRecvStrmPkt *recvStrmPkt,
                                  RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to generate response message of command message                   */
RtmpSendStrmPkt *GenerateCmdMsgRes (RtmpClient *clientPtr,
                                    RtmpRecvStrmPkt *recvStrmPkt,
                                    RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to generate response message of user control message              */
RtmpSendStrmPkt *GenerateUsrCtrlMsgRes (RtmpClient *clientPtr,
                                        RtmpRecvStrmPkt *recvStrmPkt,
                                        RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to generate response message of protocol control message          */
RtmpSendStrmPkt *GenerateProtoCtrlMsgRes (RtmpClient *clientPtr,
                                          RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to generate audio/video message                                   */
RtmpSendStrmPkt *GenerateRtmpMediaMsg (RtmpClient *clientPtr,
                                      RTMP_EXACT_MSG_TYPE exactMsgTypeId,
                                      INT32 bufferSize, UINT8 *buffer,
                                      INT32 timeStamp);


/* Function to generate data message                                          */
RtmpSendStrmPkt *GenerateRtmpDataMsg (RtmpClient *clientPtr,
                                        RtmpRecvStrmPkt *recvStrmPkt,
                                        RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to send generated message to connected client                     */
INT32 SendMsgToClient (RtmpClient *clientPtr, RtmpSendStrmPkt *sndStrmPkt,
                       RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to send data message to all connected clients                     */
INT32 SendMsgToConnectedClients (RtmpSession *rtmpSession,
                                 UINT8 *dataBuff, INT32 dataSize,
                                 INT32 deviceFd, VOID *vZeroPtr,
                                 UINT64 timeStamp,
                                 RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to write data in amf object                                       */
INT32 WriteCmdMsgPayload (RtmpClient *clientPtr, RtmpSendStrmPkt *sndStrmPkt,
                          UINT8 *cmdName, DOUBLE transactionId,
                          UINT8 *fmsVer, DOUBLE capability, DOUBLE mode,
                          UINT8 *levelVal, UINT8 *codeVal, UINT8 *descVal,
                          UINT8 *objEnc, DOUBLE strmId);

/* Function to check whether any client is connected to session or not        */
INT32 IsAnyClientConnected (RtmpSession *rtmpSession);

/* Function to add client in session                                          */
INT32 AddClientInSession (RtmpSession *rtmpSession, RtmpClient *clientPtr);

/* Function to send message to specified socket using sendmsg                 */
INT32 SendMessage (SOCKET cliSocket, RtmpData *rtmpData, INT32 dataCount,
                   RtmpSendStrmPkt *sndStrmPkt, UINT8 sendFileFlag);

/* Function to send message to specified socket using sendfile                */
INT32 SendFile (SOCKET cliSocket, RtmpData *rtmpData, INT32 rtmpDataCnt,
                   RtmpSendStrmPkt *sndStrmPkt, INT32 deviceFd, VOID *vZeroPtr);

/* Function to generate chunk header                                          */
RtmpData* GenerateChunkHdr (RtmpClient *clientPtr,
                            RtmpSendStrmPkt *sndStrmPkt,
                            RTMP_EXACT_MSG_TYPE exactMsgTypeId,
                            MESSAGE_HEADER_TYPE msgHdrType);

/* Function to generate type-0 message header                                 */
INT32 GenRtmpType0MsgHdr (RtmpSendStrmPkt *sndStrmPkt, RtmpData *outBuff);

/* Function to generate type-1 message header                                 */
INT32 GenRtmpType1MsgHdr (RtmpSendStrmPkt *sndStrmPkt, RtmpData *outBuff);

/* Function to generate type-2 message header                                 */
INT32 GenRtmpType2MsgHdr (RtmpSendStrmPkt *sndStrmPkt, RtmpData *outBuff);

/* Function to generate type-3 message header                                 */
INT32 GenRtmpType3MsgHdr (INT32 strmId, RtmpData *outBuff);

/* Function to create new RtmpClient                                          */
RtmpClient* CreateNewRtmpClient (RtmpServer *rtmpServer, CLIENT_TYPE cType,
                                 SOCKET cSocket, UINT8 *cIp, UINT16 cPort);

/* Function to destroy RtmpClient                                             */
INT32 DestroyRtmpClient (RtmpClient* rtmpClient);

/* Thread to serve connected clients on RTMP                                  */
INT32 ServeRtmpClient (VOID *threadArgs);

/* Thread to serve connected clients on RTMPT                                 */
INT32 ServeRtmptClient (VOID *threadArgs);

/* Function to add audio information in session                               */
INT32 AddAudioInfo (RtmpSession *rtmpSession, RtmpAudioInfo *adInfo);

/* Function to parse audio configuration information                          */
INT32 ParseAdConfigData (RtmpCodecInfo *codecInfo);

/* Function to add video information in session                               */
INT32 AddVideoInfo (RtmpSession *rtmpSession, RtmpVideoInfo *vdInfo);

/* Function to parse video configuration information                          */
INT32 ParseVdConfigData (RtmpCodecInfo *codecInfo);

/* Function to serve client in case of error                                  */
VOID ServeClientForError (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt,
                          INT32 intErrNo, RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to start sending data after sending pre-required rtmp messages    */
INT32 StartSendingData (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt,
                        RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to read RTMP message bytes from the given socket                  */
INT32 ReadNoOfBytes (RtmpClient *clientPtr, UINT8 *buffer, INT32 bufferSize);

/* Function to read HTTP message                                              */
INT32 ReadHttpMessage (RtmpClient *clientPtr, UINT8 *dataBuff, INT32 *dataSize);

/* Function to identify HTTP method                                           */
TUNNEL_RTMP_METHOD IdentifyMethod (UINT8 *buffer);

/* Function to find specific character in buffer                              */
INT32 FindCharInBuffer (UINT8 *buffer,  UINT8 findChar,
                         UINT8 endChar,  INT32 bufLen);

/* Function to parse HTTP request message                                     */
TunnelRTMPReqMsg* ParseHttpReqMsg (RtmpClient *clientPtr, UINT8 *dataBuff,
                                    INT32 dataSize);

/* Initialize Mutex                                                           */
INT32 RtmpMutexInit (LOCK_HANDLE *CriticalSection);

/* Function to lock critical section                                          */
INT32 RtmpMutexLock (LOCK_HANDLE *CriticalSection);

/* Function to unlock critical section                                        */
INT32 RtmpMutexUnlock (LOCK_HANDLE *CriticalSection);

/* Destroy Mutex                                                              */
INT32 RtmpMutexDestroy (LOCK_HANDLE *CriticalSection);

/* Get IP address of the given interface                                      */
INT32 RtmpGetIP (UINT8 *nwInterface, UINT8 *ipAddr);

/* Close the specified socket                                                 */
INT32 RtmpCloseSocket (SOCKET *socketId);

/* Function to create thread                                                  */
INT32 RtmpThreadCreate (THREAD_HANDLE *threadHandle, THREAD_FUNC threadFunction,
                       VOID* threadArg);

/* Function to exit the thread                                                */
INT32 RtmpThreadExit (THREAD_HANDLE threadHandle);

/* Function to detach the thread                                              */
INT32 RtmpThreadDetach (THREAD_HANDLE ThreadHandle);

/* Function to wait for thread completion                                     */
INT32 RtmpWaitForThreadCompletion (THREAD_HANDLE threadHandle);

/* Function to get self thread handle                                         */
THREAD_HANDLE RtmpGetSelfThreadHandle ();

/* Function to receive given nomber of bytes from socket                      */
INT32 RtmpReadBytes (SOCKET clientSocket, UINT8 *buffer, INT32 bufferSize);

/* Function to send given number of bytes to the socket                       */
INT32 RtmpSendBytes (SOCKET clientSocket, UINT8 *buffer, INT32 bufferSize);

/* Function to send data pointed by msg header to the socket                  */
INT32 RtmpSendMsgBytes (SOCKET clientSocket, struct msghdr *msgPtr, INT32 flags);

/* Fuction to get system time                                                 */
UINT64 RtmpGetSystemTime();

/* Function to ignore SIGPIPE in case of socket error                         */
void HandleSignal(void);

#endif  /* _INCLUDE_SERVER_COMMON_H */

/*******************************************************************************
 * Copyright (c) 2010 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * This file may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/
